/**
 * 
 * 		DMX Receive Library
 * 
 * 	Target MCU: ATtiny2313
 * 	Author: Jan Wiese <jw-lighting@ewetel.net>
 *  Date: March 2011
 *  Version 1.0
 *  License: CC 3.0 BY-NC-SA
 * 
 **/

#ifndef UBRR_VAL
	#define UBRR_VAL 1 
	// (250kbps at F_CPU 8e6, see datasheet attiny2313 p.138
	#warning "UBRR_VAL not defined. Assumed '1' (250kbps at 8MHz). DMX receive might not work."
#endif

#define CHANNEL_TO_RX	2

// DMX States
#define DS_NONE		0
#define DS_RESET	1
#define DS_STARTBYTE 	2
#define DS_DATABYTESB	10	// b -> before datarx (startaddress wasn't reached yet)
#define DS_DATARX	11
#define DS_DATABYTESA   12	// a -> after datarx (startaddress was reached)
#define DS_FAIL		20

volatile uint8_t dmx_state;
volatile uint8_t dmx_comming;
volatile uint16_t dmx_startaddr; // to be filled outside this lib
volatile uint16_t dmx_ch_counter;
volatile uint8_t DMX_RXvals[CHANNEL_TO_RX];

void dmx_init(void){
	
	UBRRH = UBRR_VAL & 0x0F00;
	UBRRL = UBRR_VAL & 0x00FF;
	
	UCSRB = (1<<RXCIE) | (1<<RXEN); // enable with interrupts
	UCSRC = (1<<USBS) | (1<<UCSZ1) | (1<<UCSZ0); // 8n2
	
	// enable timer polling for anything going on
	TCCR1B |= (1<<CS12); // run with f_cpu/256
	TIMSK |= (1<<TOIE1); // enable OVF interrupt (comming with ~0.48 Hz)
	
}

// receiver polling routine
ISR(TIMER1_OVF_vect){
	
	if(dmx_comming == 2 || dmx_comming == 0){ // stil waiting for something comming in - not ok, so no dmx incomming
		dmx_comming = 0;
		dmx_state = DS_NONE;
	}
	else{
		dmx_comming = 2; // wait status again, everything ok
	}
	
}

// receive routine
ISR(USART_RX_vect){
	
	uint8_t rxstatus = UCSRA; // read status
	uint8_t rxval = UDR; // read data and empty buffer
	
	
	if(rxstatus & (1<<FE)){
		
		if(dmx_state == DS_RESET || // more than one FE is detected as failure
		   dmx_state == DS_DATABYTESB){ // there were no receive of dmx_data. startaddr is to high/not enough channels on the line
			dmx_state = DS_FAIL;
		}
		else{
			dmx_state = DS_RESET;
		}
		return;
	}
	
	if(rxstatus & (1<<DOR)){ // dataoverrun is detected as fail
		dmx_state = DS_FAIL;
		return;
	}
	
	
	
	switch(dmx_state){
		case DS_RESET:
			
			dmx_ch_counter = 0;
			
			if(rxval == 0){ // startbyte with 0 is detected as ok. otherwise previous FE was not reset but failure
				if(dmx_startaddr == 1){ // do not wait next time, but catch first incomming value.
					dmx_state = DS_DATARX;
				}
				else{
					dmx_state = DS_STARTBYTE;
				}
				dmx_comming = 1; // valid startbyte, dmx is comming
			}
			else{
				dmx_state = DS_FAIL;
			}
		break;
		case DS_STARTBYTE:
		case DS_DATABYTESB:
			
			dmx_ch_counter++;
			
			if(dmx_ch_counter == (dmx_startaddr - 1)){ // receive next channel
				dmx_state = DS_DATARX;
			}
		break;
		case DS_DATARX:
			dmx_ch_counter++;
			
			uint8_t ch_to_rx = (uint8_t) dmx_ch_counter - dmx_startaddr;
			
			DMX_RXvals[ch_to_rx] = rxval;
			
			if(ch_to_rx >= CHANNEL_TO_RX){
				dmx_state = DS_DATABYTESA; // channels received, just wait now.
			}
		break;
		default: break;
	}
}
