/*******************************************************************
 *
 *		DMX controlled Desklight
 *		
 *  
 * 	License: CC 3.0 BY-NC-SA
 * 	Author: Jan Wiese <jw-lighting@ewetel.net>
 * 	Version: 2.0 stable
 * 	Date: 29.12.2011
 *  Target MCU: ATtiny2313
 *
 ******************************************************************/

#define F_CPU	8e6

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "dmxRx_lib_t2313.c"

#define LED_PORT	PORTD
#define LED_DDR		DDRD
#define LED_DMX		PD4
#define LED_HOLD	PD1
#define OUTPUT		PD5

#define DIP_DDR		DDRB
#define DIP_PORT	PORTB
#define DIP_PIN		PINB
#define IN_DDR		DDRD
#define IN_PORT		PORTD
#define IN_PIN		PIND
#define DIP9		PD6
#define HOLD		PD3
#define TWO_CH		PD2


#define FADE_SPEED		30
#define FADE_WAIT		70


uint8_t status;
#define ST_HOLD			0
#define ST_TWO_CH		1
#define ST_HOLD_TC		2
#define ST_HOLD_TC_UNRELEASED	3
#define ST_FADE_DIR_SET		4


void output_init(void);
void input_init(void);
uint16_t get_dmx_startaddr(void);
void set_outputs(void);
void read_inputs(void);
void check_second_ch(void);


int main(void){
	
	dmx_init();
	input_init();
	output_init();
	sei();
	
	while(1){
		dmx_startaddr = get_dmx_startaddr();
		
		check_second_ch();
		read_inputs();
		set_outputs();
		
	}
	
}

void output_init(void){
	
	LED_DDR |= (1<<LED_DMX) | (1<<LED_HOLD) | (1<<OUTPUT);
	
	OCR0B = 0xFF; // start at 0% brightness (-> 0xFF because *inverted* Fast-PWM)
	TCCR0A |= (1<<COM0B1) | (1<<COM0B0) | (1<<WGM01) | (1<<WGM00); // inverted fast-pwm at ocr0b
	TCCR0B |= (1<<CS01); // run with fcpu/8
}

void input_init(void){
	
	DIP_DDR = 0x00;
	IN_DDR &= ~(1<<DIP9);
	IN_DDR &= ~(1<<HOLD);
	IN_DDR &= ~(1<<TWO_CH);
	
	DIP_PORT = 0xFF;
	IN_PORT |= (1<<DIP9) | (1<<HOLD) | (1<<TWO_CH);
	
}

uint16_t get_dmx_startaddr(void){
	
	uint16_t tmp;
	
	tmp = (uint16_t) DIP_PIN;
	tmp = ~tmp;
	tmp &= 0x00FF;
	
	if(!(IN_PORT & (1<<DIP9))){
	    tmp |= (1<<8);
	}
	
	tmp++;
	
	return tmp;
}

void set_outputs(void){
	
	// LED outputs
	if(dmx_comming == 0 || dmx_state == DS_NONE){ // no signal comming in?
		LED_PORT &= ~(1<<LED_DMX);
	}
	else if(dmx_state == DS_FAIL){
		LED_PORT ^= (1<<LED_DMX); // flicker
		_delay_ms(300);
	}
	else{
		LED_PORT |= (1<<LED_DMX);
	}
	
	
	/// Fade up and down in stand alone (no dmx) between hold is set
	/// change direction at upper and lower end, after hold
	static uint8_t fade = 0, wait_cnt = 0;
	static int8_t fadeAdd = 1;
	if(dmx_comming == 0 || dmx_state == DS_NONE){ // no signal comming in?
		
		// fade if hold disabled
		if(!(status & (1<<ST_HOLD))){
			status |= (1<<ST_FADE_DIR_SET);
			
			// change direction at upper and lower end of scope, wait a moment
			if(fade == 0){
				fadeAdd = 0;
				wait_cnt++;
				if(wait_cnt > FADE_WAIT){
					fadeAdd = 1;
					wait_cnt = 0;
				}
			}
			else if(fade == 255){
				fadeAdd = 0;
				wait_cnt++;
				if(wait_cnt > FADE_WAIT){
					fadeAdd = -1;
					wait_cnt = 0;
				}
			}
			
			_delay_ms(FADE_SPEED);
			
			fade = (uint8_t) (fade + fadeAdd); // fade up and down
			
		}
		else{
			if(status & (1<<ST_FADE_DIR_SET)){ // only do this once
				status &= ~(1<<ST_FADE_DIR_SET);
				if(fade == 0 || fade == 255){
					wait_cnt = FADE_WAIT;
				}
				
				// change direction
				if(fadeAdd > 0){
					fadeAdd = -1;
				}
				else{
					fadeAdd = 1;
				}
			}
		}
		
		DMX_RXvals[0] = fade;
		
	}	
	
	
	if((status & (1<<ST_HOLD)) || (status & (1<<ST_HOLD_TC))){
		LED_PORT |= (1<<LED_HOLD);
	}
	else{
		LED_PORT &= ~(1<<LED_HOLD);
	}
	

	if((status & (1<<ST_HOLD)) || (status & (1<<ST_HOLD_TC))){
		return; // do not change anything
	}
	
	
	OCR0B = 0xFF - DMX_RXvals[0]; // inverted output...
	
	
}

void check_second_ch(void){
	if(!(status & (1<<ST_TWO_CH))){
		return;
	}
	
	
	if(DMX_RXvals[1] > 127){ // "key" pressed
		if(status & (1<<ST_HOLD_TC)){ // hold active
			if(!(status & (1<<ST_HOLD_TC_UNRELEASED))){ // key was released before
				status &= ~(1<<ST_HOLD_TC); // unactivate
				status |= (1<<ST_HOLD_TC_UNRELEASED); // remember pressed "key"
			}
		}
		else{ // hold unactive
			if(!(status & (1<<ST_HOLD_TC_UNRELEASED))){ // key was released before
				status |= (1<<ST_HOLD_TC); // activate
				status |= (1<<ST_HOLD_TC_UNRELEASED); // remember pressed "key"
			}
		}
	}
	else{ // key released
		status &= ~(1<<ST_HOLD_TC_UNRELEASED); // remmber unreleased
	}
}

void read_inputs(void){
	
	// get two channel status
	if(!(IN_PIN & (1<<TWO_CH))){
		status |= (1<<ST_TWO_CH);
	}
	else{
		status &= ~(1<<ST_TWO_CH);
	}
	
	// get hold status
	if(!(IN_PIN & (1<<HOLD))){
		status |= (1<<ST_HOLD);
	}
	else{
		status &= ~(1<<ST_HOLD);
	}
	
}

